#############################################################################
  
  # BayClone2 :  A Bayesian Sequence and Copy Number Caller for Subclones Using NGS Data
  # Copyright (C) <2014>  <Juhee Lee>
  # This file is part of BayClone2.

  #  BayClone2 is free software: you can redistribute it and/or modify
  #  it under the terms of the GNU General Public License as published by
  #  the Free Software Foundation, either version 3 of the License, or
  #  (at your option) any later version.

  #  BayClone2 is distributed in the hope that it will be useful,
  #  but WITHOUT ANY WARRANTY; without even the implied warranty of
  #  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  #  GNU General Public License for more details.

  #  You should have received a copy of the GNU General Public License
  #  along with BayClone2. If not, see <http://www.gnu.org/licenses/>.
  ############################################################################ 

set.seed(79861)
####SUMMARY FOR FIGURES######################
#POSTERIOR MARIGNAL DIST OF C
post_dist <- rep(0, n_C)
for(i.c in Min_C:Max_C) post_dist[i.c-1] <- sum(MCMC.sam$C==i.c)/n.sam

##PLOT THE MARIGNAL DISTRIBUTION OF C
pdf("BayClone2_sim1_post_C.pdf")
par(mar=c(4.5, 4.5, 2.1, 2.1))
plot((1:(Max_C-1)), post_dist, type="o", lwd=4, col=1, xlab="C", ylab="P(C|n, N)", main="", cex.axis=1.5, cex.lab=1.5)
abline(v=(sim.tr$C-1), lty=2, lwd=3, col=2)
dev.off()


#################################################################
####WE FIND POSTERIOR POINT ESTIMATES OF L, Z, W, PHI, PI, P0
##################################################################
library(combinat)
dyn.load("../source/BayClone2_post.so")

CC <- which.max(post_dist)+1  ###POSTERIOR MODE OF C, C_HAT--INCLUDING THE BACKROUND SUBCLONE

###################################################################
###COMPUTE DISTANCE AMONG L (SAMPLED THROUGH MCMC)
dist.C <- NULL
sam.C <- (1:n.sam)[MCMC.sam$C==CC]
n.C <- sum(MCMC.sam$C==CC)
D.C <- array(0, dim=c(n.C, n.C))

ind_N <- factorial(CC-1)
index_set <- array(NA, dim=c(ind_N, CC-1))
tmp <- permn(CC-1)
for(i.n in 1:ind_N)
{
    index_set[i.n,] <- tmp[[i.n]]
}

index_set <- array(index_set-1, dim=c(1, ind_N*(CC-1)))[1,]

print(paste("n.C=", n.C))
for(i in 1:(n.C-1))
{
    if((i%%100)==0)
    {
        print(paste("i.sam=", i))
        print(date())
    }
    
    for(j in (i+1):n.C)
    {
        i.sam <- sam.C[i]
        j.sam <- sam.C[j]
        
        L.i <- array(MCMC.sam$L[[i.sam]], dim=c(1, S*CC))[1,]
        L.j <- array(MCMC.sam$L[[j.sam]], dim=c(1, S*CC))[1,]
        
        output <- .C("fn_sum_Z_1", SS=as.integer(S), CC=as.integer(CC-1), ind_NN=as.integer(ind_N), ind_set=as.integer(index_set), Z_1=as.integer(L.i), Z_2=as.integer(L.j), min_dist=as.double(0))
        
        D.C[i, j] <- D.C[j, i] <- output$min_dist
    }
}

dist.C[[CC]] <- D.C
#################################################################################################################
###FIND THE SAMPLE OF L MINIMIZING THE DISTANCE FROM ALL THE OTHER SAMPLES OF L
dist.tmp <- dist.C[[CC]]
sam.C <- (1:n.sam)[MCMC.sam$C==CC]
sum.dist <- apply(dist.tmp, 1, sum)
ind.min <- which.min(sum.dist)
ind.min <- sam.C[ind.min]

#####################################################
post.L <- MCMC.sam$L[[ind.min]]  ### POSTERIOR POINT ESTIMATE OF L, L_HAT
post.Z <- MCMC.sam$Z[[ind.min]]  ### POSTERIOR POINT ESTIMATE OF Z, Z_HAT
post.w <- MCMC.sam$w[[ind.min]]  ### POSTERIOR POINT ESTIMATE OF W, W_HAT


###YOU MAY NEED TO RE-AGRRANGE THE COLUMNS (SUBCLONES) FOR EASIER COMPARISON
#####CHECK THE POINT ESTIMATES BY EYES AND RE-ARRANGE MANUALLY
#post.L <- post.L[,order(c(1, 5, 4, 2, 3))]
#post.Z <- post.Z[,order(c(1, 5, 4, 2, 3))]
#post.w <- post.w[,order(c(1, 5, 4, 2, 3))]


post.p0.z <- mean(MCMC.sam$p0_z[sam.C])  ### POSTERIOR POINT ESTIMATE OF P0, P0_HAT
post.phi <- apply(MCMC.sam$phi[sam.C,], 2, mean)  ### POSTERIOR POINT ESTIMATE OF PHI, PHI_HAT

#####################################################
####COMPUTE ESTIMATES OF M_ST AND P_ST CONDITIONAL ON C_HAT
#####################################################
p_st_post <- M_st_post <- array(0, dim=c(S, T))

for(i.sam in sam.C)
{
    phi_tmp <- MCMC.sam$phi[i.sam,]
    M_tmp <- MCMC.sam$M[i.sam,,]
    p_tmp <- MCMC.sam$p[i.sam,,]
    
    M_st_post <- M_st_post + M_tmp
    p_st_post <- p_st_post + p_tmp
}

M_st_post <- M_st_post/length(sam.C)
p_st_post <- p_st_post/length(sam.C)

#####################################################
#DIFFERENCE IN M_ST  (POSTERIOR ESTIMATE OF M_ST - TRUE M_ST)
pdf("BayClone2_sim1_diff_M.pdf")
par(mar=c(4.5, 4.5, 2.1, 2.1))
hist(M_st_post - sim.tr$M, lwd=4, xlab="M^hat - M^true", main="", cex.axis=1.5, cex.lab=1.5, col="grey")
abline(v=0, lty=2, col=2, lwd=2)
dev.off()

#DIFFERENCE IN P_ST (POSTERIOR ESTIMATES OF P_ST - TRUE P_ST)
min_p <- min(p_st_post - sim.tr$p)-0.05  #MIN OF THE GRID OF THE HISTROGRAM
max_p <- max(p_st_post - sim.tr$p)+0.05  #MAX OF THE GRID OF THE HISTROGRAM

pdf("BayClone2_sim1_diff_p.pdf")
par(mar=c(4.5, 4.5, 2.1, 2.1))
hist(p_st_post - sim.tr$p, lwd=4, xlab="p^hat - p^true", main="", cex.axis=1.5, cex.lab=1.5, col="grey", breaks=seq(min_p, max_p, by=0.03))
abline(v=0, lty=2, col=2, lwd=2)
dev.off()



#######################################
###############HEATMAPS FOR ILLUSTRATION
#####CAUTION: HEATMAPS CAN BE DONE FOR C-HAT >=2!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#####TO MAKE HEATMAPS, PLEASE COMMENT IN THE LINES BELOW
#######################################
#library(gplots);
#lmat = rbind(c(0,3),c(2,1),c(0,4))
#lwid = c(1.5,4)
#lhei = c(1.5,4,1.2)

####HEATMAP OF L-HAT
#pdf("BayClone2_sim1_L.pdf")
#colnames(post.L) <- (0:(CC-1))
#heatmap.2(post.L[,-1], trace="none", Colv=FALSE, Rowv=FALSE, dendrogram="none", scale="none", col=redgreen, colsep=(1:(CC-1)), sepcol=c("white", "white"), sepwidth=c(0.05, 0.1), main="",lmat = lmat, lwid = lwid, lhei = lhei)
#dev.off()

####HEATMAP OF TRUE L
#pdf("BayClone2_sim1_L_tr.pdf")
#L.tr <- sim.tr$L
#colnames(L.tr) <- (0:(sim.tr$C-1))
#heatmap.2(L.tr[,-1], trace="none", Colv=FALSE, Rowv=FALSE, dendrogram="none", scale="none", col=redgreen, colsep=(1:(sim.tr$C-1)), sepcol=c("white", "white"), sepwidth=c(0.05, 0.1), main="",lmat = lmat, lwid = lwid, lhei = lhei)
#dev.off()


#MATCHING THE COLOER KEYS IN THE HEATMAPS OF Z_HAT AND TRUE Z
#rg <- redgreen(100)      # the original color vector
#Z.tr <- sim.tr$Z
#min.Z <- min(min(post.Z), min(Z.tr))
#max.Z <- max(max(post.Z), max(Z.tr))

#rg.post <- rg[round((min(post.Z)/max.Z - min.Z)*100):round(100*max(post.Z)/(max.Z - min.Z))]
#rg.tr <- rg[round(min(Z.tr)/(max.Z - min.Z)*100):round(100*max(Z.tr)/(max.Z - min.Z))]

####HEATMAP OF Z-HAT
#pdf("BayClone2_sim1_Z.pdf")
#colnames(post.Z) <- (0:(CC-1))
#heatmap.2(post.Z[,-1], trace="none", Colv=FALSE, Rowv=FALSE, dendrogram="none", scale="none", col=rg.post, colsep=(1:(CC-1)), sepcol=c("white", "white"), sepwidth=c(0.05, 0.1), main="",lmat = lmat, lwid = lwid, lhei = lhei)
#dev.off()

####HEATMAP OF TRUE Z
#pdf("BayClone2_sim1_Z_tr.pdf")
#colnames(Z.tr) <- (0:(sim.tr$C-1))
#heatmap.2(Z.tr[,-1], trace="none", Colv=FALSE, Rowv=FALSE, dendrogram="none", scale="none", col=rg.tr, colsep=(1:(sim.tr$C-1)), sepcol=c("white", "white"), sepwidth=c(0.05, 0.1), main="",lmat = lmat, lwid = lwid, lhei = lhei)
#dev.off()



#MATCHING THE COLOER KEYS IN THE HEATMAPS OF W_HAT AND TRUE W
#rg <- redgreen(100)      # the original color vector
#w.tr <- sim.tr$w
#min.w <- min(min(post.w), min(w.tr))
#max.w <- max(max(post.w), max(w.tr))

#rg.post <- rg[round((min(post.w)/max.w - min.w)*100):round(100*max(post.w)/(max.w - min.w))]
#rg.tr <- rg[round(min(w.tr)/(max.w - min.w)*100):round(max(w.tr)/(max.w - min.w)*100)]

####HEATMAP OF TRUE W
#pdf("BayClone2_sim1_w_tr.pdf")
#colnames(w.tr) <- (0:(sim.tr$C-1))
#heatmap.2(w.tr, trace="none", Colv=FALSE, Rowv=TRUE, dendrogram="none", scale="none", col=rg.tr, colsep=(1:(sim.tr$C-1)), sepcol=c("white", "white"), sepwidth=c(0.05, 0.1), main="",lmat = lmat, lwid = lwid, lhei = lhei)
#dev.off()

#tmp.w.tr.heatmap <- heatmap.2(w.tr, trace="none", Colv=FALSE, Rowv=TRUE, dendrogram="none", scale="none", col=rg.tr, colsep=(1:(sim.tr$C-1)), sepcol=c("white", "white"), sepwidth=c(0.05, 0.1), main="",lmat = lmat, lwid = lwid, lhei = lhei)
#w_tmp <- post.w[tmp.w.tr.heatmap$rowInd,]

####HEATMAP OF W-HAT
#pdf("BayClone2_sim1_w.pdf")
#colnames(w_tmp) <- (0:(CC-1))
#rownames(w_tmp) <- tmp.w.tr.heatmap$rowInd
#heatmap.2(w_tmp, trace="none", Colv=FALSE, Rowv=FALSE, dendrogram="none", scale="none", col=rg.post, colsep=(1:(CC-1)), sepcol=c("white", "white"), sepwidth=c(0.05, 0.1), main="",lmat = lmat, lwid = lwid, lhei = lhei)
#dev.off()

